import gtk
import gtk.gdk
import logging
from SynologyCommand import SynologyCommand
from downloadredirector import PreferencesDownloadredirectorDialog
class Droplet(gtk.Window):
    """
    Draws a small window with no decorations (only a pixmap)
    where you can drop stuff.
    """

    def __init__(self,syno_command):
        #FIXME : Make this window a widget or whatever it's called,
        # it shouldn't be in the windows list in gnome-panel
    
        self.syno_command = syno_command
        gtk.Window.__init__(self)
        
        self.init_prefs()
        
        if "droplet_x" in self.preferences and "droplet_y" in self.preferences:
            x = self.preferences["droplet_x"]
            y = self.preferences["droplet_y"]
        else:
            x = 50
            y = 50
        self.move(x,y)
        self.drag_dest_set(gtk.DEST_DEFAULT_DROP, [("text/plain",0,0)], gtk.gdk.ACTION_COPY)
        self.set_events(self.get_events() | gtk.gdk.BUTTON_PRESS_MASK)
        self.connect("button_press_event", self.mouse_callback)
        self.connect('drag_motion', self.motion_callback)
        self.connect("drag_drop", self.dropped_callback)
        self.connect("drag_data_received", self.receive_callback)
        self.set_decorated(False)
        self.show()
        
        pixmap, mask = gtk.gdk.pixmap_create_from_xpm(
            self.window, None, "data/media/droplet.xpm")
        image = gtk.Image()
        image.set_from_pixmap(pixmap, mask)
        image.show()
        
        fixed = gtk.Fixed()
        fixed.set_size_request(100, 100)
        fixed.put(image, 0, 0)
        self.add(fixed)
        fixed.show()

        # This masks out everything except for the image itself
        self.shape_combine_mask(mask, 0, 0)
        
        logging.debug( "Setting up the menu" )
        self.menu = gtk.Menu()
        openItem = gtk.MenuItem("_Open Download Redirector")
        self.menu.append(openItem)

        showItem = gtk.MenuItem("_Enable Drop Zone")
        self.menu.append(showItem)

        hideItem = gtk.MenuItem("_Disable Drop Zone")
        self.menu.append(hideItem)

        exitItem = gtk.MenuItem("E_xit")
        self.menu.append(exitItem)

        openItem.show()
        showItem.show()
        hideItem.show()
        exitItem.show()
        
        exitItem.connect_object("activate", gtk.main_quit, "file.quit")
        hideItem.connect_object("activate",self.hide_window,None)
        showItem.connect_object("activate",self.show_window,None)

        #Keep always on top
        self.set_keep_above(True)
        
    def init_prefs(self):
        self.pref_dlg = PreferencesDownloadredirectorDialog.NewPreferencesDownloadredirectorDialog()
        self.preferences = self.pref_dlg.get_preferences()

    def motion_callback(self,wid, context, x, y, time):
        context.drag_status(gtk.gdk.ACTION_COPY, time)
        return True

    def mouse_callback(self,*kwargs):
        """Move the drop box around the screen or display the menu"""
        ev = kwargs[1]
        if not hasattr(ev,"button"):
            return True

        if ev.button == 1:
            self.begin_move_drag(1,int(ev.x_root),int(ev.y_root),ev.time)
            self.preferences["droplet_x"] = int(ev.x_root)
            self.preferences["droplet_y"] = int(ev.y_root)
            self.pref_dlg.save()
            self.pref_dlg = None
            self.init_prefs()
        if ev.button == 3:
            #Display the menu
            self.menu.popup(None,None,None,button=3,activate_time=ev.time)
       
    def dropped_callback(self, widget,context, x, y, time):
        """Function called when something is dropped in the drop box"""
        context.finish(True, False, time)
        return False

    def receive_callback(self, widget, context, x, y, data, info, time):
        data_string = data.data
        logging.debug(data_string)
        if data_string.startswith("http://") or data_string.startswith("file://"):
            self.syno_command.post_torrent(data_string)

    def hide_window(self,widget):
        self.hide()

    def show_window(self,widget):
        self.show()

if __name__ == "__main__":
    syno_command = SynologyCommand("newport")
    
    droplet = Droplet(syno_command)
    

