import json
import urllib
import urllib2
import httplib
import mimetypes

class SynologyCommand:
    """
    Send commands to Synology NAS
    """
    def __init__(self,ip, port="5000"):
        self.port = port 
        self.host = ip + ":" + self.port
        self.cgi = "/download/download_redirector.cgi"
        self.url = "http://" + self.host + self.cgi
        
    def login(self,login,password):
        self.username = login     
        synology_settings = {'username' : login,
          'passwd' : password,
          'action' : 'login' }
        post_data = urllib.urlencode(synology_settings)
        req = urllib2.Request(self.url, post_data)
        
        try:
            response = urllib2.urlopen(req)
        except:
             print "Server at", self.host ,"is not reachable"
             return False
        synology_response_data = response.read()
        self.login_data = json.loads(synology_response_data)
        
        if self.login_data['login_success'] == True:
            self.id = str(self.login_data["id"])
            return True
        else:
            print "Login fail"
            return False
         
    def get_conf(self):
        data = {'id' : self.id,
          'action' : 'getconf' }
        reply = self.send_post(data)
        return reply
    
    def get_shares(self):
        data = { 'id' : self.id,
                'action' : 'rwshareget',
                'username' : self.username }
        return self.send_post(data)
        
    def get_all(self):
        """
        Get list of all downloads on the server
        Note about the status field on a item: 
         - 1 : Waiting
         - 2 : Downloading
         - 3 : Paused
         - 4 : ?
         - 5 : Completed
         - 6 : ? 
         - 7 : ?
         - 8 : Seeding
        """
        data = {'id' : self.id,
                'action' : 'getall'}
        reply = self.send_post(data)
        for item in reply["items"]:
            if item["status"] == 1:
                item["status"] = "Waiting"
            elif item["status"] == 2:
                item["status"] = "Downloading"
            elif item["status"] == 3:
                item["status"] = "Paused"
            elif item ["status"] == 5:
                item["status"] = "Completed"
            elif item["status"] == 6:
                item["status"] = "Checking"
            elif item ["status"] == 8:
                item["status"] = "Seeding"
            #item["currentRate"] = "%.2f KB/s" % (item["currentRate"] / 1000.0)
            #item["uploadRate"] = "%.2f KB/s" % (item["uploadRate"] / 1000.0)
                
        return reply
    
    def pause(self,idList):
        """Pause selected download"""
        data = { 'id' : self.id,
                'action' : "stop",
                'idList' : idList }
        return self.send_post(data)
        
    def resume(self,idList):
        """Resume selected download"""
        data = { 'id' : self.id,
                'action' : "resume",
                'idList' : idList }
        return self.send_post(data)
    
    def delete(self,idList):
        """Delete selected download"""
        data = { 'id' : self.id,
                'action' : "delete",
                'idList' : idList }
        return self.send_post(data)

    def clear_finished(self):    
        """Clear all finished downloads"""
        data = { 'id' : self.id,
                'action' : 'clear',
                'username' : self.username }
        return self.send_post(data)
    
    def send_post(self,data):
        post_data = urllib.urlencode(data)
        request = urllib2.Request(self.url, post_data)
        response = urllib2.urlopen(request)
        synology_response_data = response.read()
        server_reply = json.loads(synology_response_data)
        return server_reply
    
    def post_torrent(self, url):
        if url.startswith("http://"):
            torrent = self.read_web(url)
        elif url.startswith("file://"):
            torrent = self.read_file(url)
        files = [("torrent",torrent["filename"],torrent["content"])]
        fields = [( "id",  self.id )]
        content_type, body = self.encode_multipart_formdata(fields, files)
        h = httplib.HTTP(self.host)
        h.putrequest('POST', self.cgi)
        h.putheader('content-type', content_type)
        h.putheader('content-length', str(len(body)))
        h.endheaders()
        h.send(body)
        errcode, errmsg, headers = h.getreply()
        return h.file.read()
    
    def read_file(self,path):
        path = path[7:]
        path = path.strip("\r\n")
        filename = path.split('/')[-1]
        file = open(path,"r")
        content = file.read()
        file.close()
        return {"filename" : filename , "content"  : content}
        
    
    def read_web(self,url):
        """Copy the contents of a file from a given URL
        to a local file.
        """
        
        webFile = urllib.urlopen(url)
        filename = url.split('/')[-1]
        content = webFile.read()
      
        webFile.close()
        webfile = {"filename": filename, "content" : content }
        return webfile
    
    def encode_multipart_formdata(self,fields, files):
        """
        fields is a sequence of (name, value) elements for regular form fields.
        files is a sequence of (name, filename, value) elements for data to be uploaded as files
        Return (content_type, body) ready for httplib.HTTP instance
        """
        BOUNDARY = '------------------------------cdcbe1e40968'
        CRLF = '\r\n'
        L = []
        for (key, value) in fields:
            L.append('--' + BOUNDARY)
            L.append('Content-Disposition: form-data; name="%s"' % key)
            L.append('')
            L.append(value)
        
        for (key, filename, value) in files:
            L.append('--' + BOUNDARY)
            L.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (key, filename))
            L.append('Content-Type: %s' % self.get_content_type(filename))
            L.append('')
            L.append(value)
        L.append('--' + BOUNDARY + '--')
        L.append('')
       
        body = CRLF.join(L)
        content_type = 'multipart/form-data; boundary=%s' % BOUNDARY
        return content_type, body
    
    def get_content_type(self,filename):
        return mimetypes.guess_type(filename)[0] or 'application/octet-stream'
    
            
        
